from gpt_index import GPTSimpleVectorIndex, SimpleDirectoryReader, PromptHelper
from langchain import OpenAI
from gpt_index import MockLLMPredictor, LLMPredictor
from IPython.display import Markdown, display
import wikipediaapi
import streamlit as st
import os

#### YOU NEED TO HAVE OPENAI_API_KEY IN YOUR ENVIRONMENT VARIABLES ####
#### !!BOTH THE EMBEDDING PROCESS AND THE QUERYING WILL USE TOKENS!! ####

wiki_wiki = wikipediaapi.Wikipedia(
        language='en',
        extract_format=wikipediaapi.ExtractFormat.WIKI
)



# create folders data, index, to_be_embedded. create tokens.txt and write 0 to it
if not os.path.exists('data'):
    os.mkdir('data')
if not os.path.exists('index'):
    os.mkdir('index')
if not os.path.exists('to_be_embedded'):
    os.mkdir('to_be_embedded')

if not os.path.exists('tokens.txt'):
    with open('tokens.txt', 'w') as f:
        f.write("0")


# define LLM
llm_predictor = LLMPredictor(llm=OpenAI(temperature=0, model_name="text-davinci-003"))

# set maximum input size
max_input_size = 1500
# set number of output tokens
num_output = 256
# set maximum chunk overlap
max_chunk_overlap = 0
prompt_helper = PromptHelper(max_input_size, num_output, max_chunk_overlap)


def wiki_search(user_input):
    # search the data folder and make a list of all the files strip the .json
    files = os.listdir('data')
    files_json = [file.replace('.json', '') for file in files]
    page_py = wiki_wiki.page(user_input)
    title = page_py.title
    title = title.lower()
    print(title)

    # search the list of files for the user input
    if title in files_json:
        # if found, load the index
        index = GPTSimpleVectorIndex.load_from_disk(f'index/{title}.json')

    elif title not in files_json:
        print("Not found")
        # if not found, search wikipedia and save the text to a file
        
        
        text = page_py.text
        word_count = len(text.split())
        print("Word count: ", word_count)
        # save text to file as .Json with the titlet. utf-8 encoding. use data folder
        with open(f"data/{title}.json", "w", encoding="utf-8") as f:
            f.write(text)
        # save the file to to_be_embedded folder
        with open(f"to_be_embedded/{title}.json", "w", encoding="utf-8") as f:
            f.write(text)
        # embed the text in the index
        # check if the index exists
        if os.path.exists(f'index/{text}.json'):
            print("Index exists")
            index = GPTSimpleVectorIndex.load_from_disk(f'index/{title}.json')
        else:
            print("Embedding text in index...")
            documents = SimpleDirectoryReader('to_be_embedded').load_data()
            print(len(documents))
            print("starting index...")
            index = GPTSimpleVectorIndex(documents)
            index.save_to_disk(f'index/{title}.json')
        # clear the to_be_embedded folder
        files = os.listdir('to_be_embedded')
        for file in files:
            os.remove(f'to_be_embedded/{file}')
    return index
# wiki_search("cpu")
# take user input
st.info('You will be interacting with AI generated content which can be inaccurate and biased. By using this app you agree that you understand  AI content can be inaccurate and biased', icon="ℹ️")
st.header(':blue[TALK TO WIKIPEDIA WITH GPT-3] :sunglasses:')
placeholder_1 = st.empty()
user_input = placeholder_1.text_input("What do you want to talk about?", key="1")
if user_input != "":
    index = wiki_search(user_input)
    st.subheader(f"You are now talking with Wikipedia article about '{user_input}' ")

    # take a question from the user
    placeholder_2 = st.empty()
    question = placeholder_2.text_input("Ask a question about the topic", key="2")
    # st.write("You asked: ", question)
    # query the index
    # response = ''
    with st.spinner('Wait for it...'):
        if question != "":
            print("Querying index...")
            response = index.query(question, llm_predictor=llm_predictor) # use llm_predictor if you want to use token counts on line 115

    
           
            # open tokens.txt read the number and add llm.last_token_usage to it
            with open('tokens.txt', 'r') as f:
                tokens = int(f.read())
                tokens += llm_predictor.last_token_usage # llm_predictor is defined on line 63
            st.write(f"TOTAL TOKENS USED BY USERS: {tokens}")
            # write the new number to tokens.txt
            with open('tokens.txt', 'w') as f:
                f.write(str(tokens))

            st.write(response)
            
